//+------------------------------------------------------------------+
//|                                            CBoundaryFragment.mqh |
//|                                Copyright 2019, Korotchenko Roman |
//|                        https://login.mql5.com/en/users/Solitonic |
//|                                             Revision 12 May 2019 |
//+------------------------------------------------------------------+
#property copyright "Copyright 2019, Korotchenko Roman"
#property link        "https://login.mql5.com/en/users/Solitonic"
#include <FRACTAL\CXY_Array.mqh>
//---------------------------------------------------------------------------------------------
// NumDivide - number of required partitions = NUMBER OF USED ELEMENTS for the ARRAY OF BOUNDARIES x[],y[]
// x[] - time series
// leftIdx, rightIdx - the left and right boundary points in the x[] array
// hSampling - discretization between points in hours
//
// RESULTS
// bnd[][2] - array of interval borders
//    nBnd     - number of points in the sub-interval
//    bnd[][1] - left boundaries in the sub-interval
//    bnd[][2] - tight boundaries in the sub-interval
//    
// Function result - duration of each sub interval in time, in days
//---------------------------------------------------------------------------------------------

double fBoundaryArray(const int NumDivide, const double &x[], const int leftIdx, const int rightIdx,  const double hSampling, CXY_Array &bnd, int &nBnd)
{
  int np, minIdx, maxIdx, flagSer = 0;
  int mb;
  double hDuration; // duration of selected sub intervals 
  
  if (rightIdx < leftIdx ){  
      flagSer = 1;  // indexes in the time series in the reverse order, from right to left
      minIdx = rightIdx;
      maxIdx = leftIdx;        
  }
  else { minIdx = leftIdx;     maxIdx = rightIdx; } // indexes for a series in a normal order
  
  np = (maxIdx - minIdx)+1; // The number of points in the provided segment including boundary points
  
  // parity check
  // if( fmod(np,2) != 0.)   np += 1;
      
      mb = bnd.length;    

  if( mb < NumDivide ) // if necessary, increase the array for boundary points
  {
    bnd.Resize(NumDivide, 16*NumDivide);    
  }
  
  bnd.FillByEmpty();         // EMPTY_VALUE 
  
  nBnd = np/NumDivide; // number of points in one division sub-interval 
  
  for( int i=0; i<NumDivide; i++)
  {
     if (!flagSer){
       bnd.x[i] = maxIdx - (i+1)*nBnd+1; // "left"  point of the sub-interval
       bnd.y[i] = bnd.x[i] + nBnd - 1;   // "right"  point of the sub-interval
     }
     else { // RIGHT-TO-LEFT ORDER
       bnd.y[i] =   maxIdx - i*nBnd;
       bnd.x[i] = bnd.y[i] - nBnd + 1;
     }
  i=i;
  }

hDuration = nBnd * hSampling;
return hDuration;
}
