//+------------------------------------------------------------------+
//|                                          CConfidenceInterval.mqh |
//|                                Copyright 2019, Korotchenko Roman |
//|                        https://login.mql5.com/en/users/Solitonic |
//|                                             Revision 12 May 2019 |
//+------------------------------------------------------------------+
#property copyright   "Copyright 2019, Roman Korotchenko"
#property link        "https://login.mql5.com/en/users/Solitonic"
#property version   "1.0"
#include <FRACTAL\SimpleStat.mqh>


double  fCalculateConfidenceIntervalMU(const double &x[], double &y[], int Count, double &abnd[], double &R2det, double alfa = 0.05)
{
//---------------------------------------------------------------------------------------------
// x[] - LogDeltaScales scale of partitioning
// y[] - logarithm of amplitude variations
// Count - number of points in arrays by which parameters of the approximating line will be estimated
// alfa - level of significance of the confidence interval (Value as % of confid.interval = 100*(1-alfa) = 95% default)
//
// RESULT - fractal index (Mu)
// bnd[] - lower and upper BOUNDARIES OF THE CONFIDENCE INTERVAL
// R2det - coefficient of determination: the closer to 1, the more accurately the calculation points fall on the approximating line
//---------------------------------------------------------------------------------------------

double Mu, A, B, DeltaA;
int idx0, idx1;
  
   idx0 = 0;           // BOUNDARY INDEXES OF THE ARRAY
   idx1 = Count;
   
    R2det = SelectingBest(x,y, idx0, idx1, A, B);
   DeltaA = ConfidenceInterval(x,y, idx0,idx1, A, B, alfa);
   
        Mu = -A;          // FRACTAL INDEX
   abnd[0] = Mu - DeltaA; // BORDERS OF THE CONFIDENCE INTERVAL
   abnd[1] = Mu + DeltaA;   
     Count = idx1-idx0;
   return Mu;
}
//============================================================================

double SelectingBest(const double &x[], const double &y[], int &idx0, int &idx1, double &Coeff,double &BB)
{
//+------------------------------------------------------------------+
//| Selecting the best borders of estimate of linear regression by   |
//| value (maximum) of the coefficient of determination              |
//| INPUT PARAMETERS:                                                |
//|     x[]     -   abscissa values                                  |
//|     y[]     -   function values (ordinates)                      |
//|     idx0    -   initial index of the interval                    |
//|     idx1    -   finishing index of the interval                  |
//|                 if the index is ZERO                             |
//|                 all f[] values are processed                     |
//| RESULT -  coefficient of determination                           |
//|     idx0    -   initial index of the interval                    |
//|     idx1    -   finishing index of the interval                  |
//|     Coeff   -   coefficient of the line Y=Coeff*X + B            |
//|        BB   -   free member                                      |                        |
//+------------------------------------------------------------------+
 double A, B, R2detMax, R2detEstim, R2det;
 int i0, i1, i0best, i1best, LIM = 3;
  
  if(idx1 == 0) idx1 = ArraySize(y)-1;
 
  i0 = i0best = idx0;
  i1 = i1best = idx1;
   R2detMax = 0;
   A = -10;
 // elements of higher indexes store results for the roughest coverage scale, in which the largest rectangles are applied
   for( int i=0; i <= LIM; i++)
   {  
      i1 -=i; 
      if( i1-idx0<4) break;
      
      R2detEstim = fLinearApprox (x,y,i0best, i1, A, B);
      if( R2detMax < R2detEstim )
      { // save the estimate and the range border
         R2detMax = R2detEstim;         
           i1best = i1;
      }
   }
// elements of lower indexes store results for the smallest coverage scales, in which the smallest rectangles are applied
   for( int i=0; i <=LIM; i++)
   {  
      i0 +=i; 
      if( i1best-i0<4) break;
      
      R2detEstim = fLinearApprox (x,y,i0, i1best, A, B);
      if( R2detMax < R2detEstim )
      { // save the estimate and the range border
         R2detMax = R2detEstim;
           i0best = i0;         
      }
      
   }
   
   R2det= fLinearApprox (x,y,i0best, i1best, A, B);
   idx0 = i0best;
   idx1 = i1best;
  Coeff = A; BB = B;
 return R2det;
}
//---------------------------------------------------------------------------------------

